@@ -9,6 +9,14 @@ import urllib
 import warnings
 from datetime import datetime as dt,timedelta
 
+try:
+    import shapefile
+    import zipfile
+    from io import StringIO, BytesIO
+except:
+    warn_message = "Warning: The libraries necessary for online NHC forecast retrieval aren't available (shapefile, gzip, io)."
+    warnings.warn(warn_message)
+
 try:
     import cartopy.feature as cfeature
     from cartopy import crs as ccrs
@@ -21,6 +29,7 @@ except:
 from .storm import RealtimeStorm
 from ..utils import *
 from .. import constants
+from ..tracks.plot import TrackPlot
 
 class Realtime():
     
@@ -154,9 +163,45 @@ class Realtime():
             if hours_diff <= -48.0:
                 del self.data[key]
         
+        #Remove invests that have been classified as TCs
+        all_keys = [k for k in self.data.keys()]
+        for key in all_keys:
+            
+            #Only keep invests
+            if self.data[key]['invest'] == False: continue
+            
+            #Iterate through all storms
+            match = False
+            for key_storm in all_keys:
+                if self.data[key_storm]['invest'] == True: continue
+                
+                #Check for overlap in lons
+                if self.data[key_storm]['lon'][0] == self.data[key]['lon'][0] and self.data[key_storm]['date'][0] == self.data[key]['date'][0]: match = True
+            
+            if match == True: del self.data[key]
+        
         #For each storm remaining, create a Storm object
         if len(self.data) > 0:
+            self.__read_nhc_shapefile()
+            
+            #Add probability attributes for storms where it's unavailable
             for key in self.data.keys():
+                if key[0:2] in ['AL','EP'] and 'prob_2day' not in self.data[key].keys():
+                    if int(key[2]) == 9:
+                        self.data[key]['prob_2day'] = '0%'
+                        self.data[key]['prob_5day'] = '0%'
+                        self.data[key]['risk_2day'] = 'Low'
+                        self.data[key]['risk_5day'] = 'Low'
+                    else:
+                        self.data[key]['prob_2day'] = 'N/A'
+                        self.data[key]['prob_5day'] = 'N/A'
+                        self.data[key]['risk_2day'] = 'N/A'
+                        self.data[key]['risk_5day'] = 'N/A'
+                if key[0:2] not in ['AL','EP']:
+                    self.data[key]['prob_2day'] = 'N/A'
+                    self.data[key]['prob_5day'] = 'N/A'
+                    self.data[key]['risk_2day'] = 'N/A'
+                    self.data[key]['risk_5day'] = 'N/A'
                 self[key] = RealtimeStorm(self.data[key])
 
             #Delete data dict while retaining active storm keys
@@ -437,11 +482,7 @@ class Realtime():
                 f = urllib.request.urlopen(url,context=ssl._create_unverified_context())
             else:
                 f = urllib.request.urlopen(url)
-            content = f.read()
-            content = content.decode("utf-8")
-            content = content.split("\n")
-            content = [(i.replace(" ","")).split(",") for i in content]
-            f.close()
+            content = read_url(url)
 
             #iterate through file lines
             for line in content:
@@ -531,18 +572,90 @@ class Realtime():
             #Add storm name
             self.data[stormid]['name'] = name
             
-    def list_active_storms(self):
+    def __read_nhc_shapefile(self):
+        
+        try:
+            
+            #Read in shapefile zip from NHC
+            url = 'https://www.nhc.noaa.gov/xgtwo/gtwo_shapefiles.zip'
+            request = urllib.request.Request(url)
+            response = urllib.request.urlopen(request)
+            file_like_object = BytesIO(response.read())
+            tar = zipfile.ZipFile(file_like_object)
+
+            #Get file list (points, areas)
+            members = '\n'.join([i for i in tar.namelist()])
+            nums = "[0123456789]"
+            search_pattern = f'gtwo_points_202{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}.shp'
+            pattern = re.compile(search_pattern)
+            filelist = pattern.findall(members)
+            files = []
+            for file in filelist:
+                if file not in files: files.append(file.split(".shp")[0]) #remove duplicates
+
+            #Retrieve necessary components for shapefile
+            members = tar.namelist()
+            members_names = [i for i in members]
+            data = {'shp':0,'dbf':0,'prj':0,'shx':0}
+            for key in data.keys():
+                idx = members_names.index(files[0]+"."+key)
+                data[key] = BytesIO(tar.read(members[idx]))
+
+            #Read in shapefile
+            orig_reader = shapefile.Reader(shp=data['shp'], dbf=data['dbf'], prj=data['prj'], shx=data['shx'])
+            shp = BasicReader(orig_reader)
+
+            #Iterate through all areas to match to existing invests
+            for record, point in zip(shp.records(), shp.geometries()):
+
+                #Read relevant data
+                lon = (list(point.coords)[0][0])
+                lat = (list(point.coords)[0][1])
+                prob_2day = record.attributes['PROB2DAY']
+                prob_5day = record.attributes['PROB5DAY']
+                risk_2day = record.attributes['RISK2DAY']
+                risk_5day = record.attributes['RISK5DAY']
+                
+                #Match to existing invests
+                distances = [great_circle((lat,lon),(self.data[storm_id]['lat'][-1],self.data[storm_id]['lon'][-1])).miles for storm_id in self.data.keys()]
+                min_distance = np.min(distances)
+                idx = distances.index(min_distance)
+                storm_id = [k for k in self.data.keys()][idx]
+                if min_distance <= 150:
+                    self.data[storm_id]['prob_2day'] = prob_2day
+                    self.data[storm_id]['prob_5day'] = prob_5day
+                    self.data[storm_id]['risk_2day'] = risk_2day
+                    self.data[storm_id]['risk_5day'] = risk_5day
+            
+        except:
+            
+            msg = "Error in retrieving NHC invest data."
+            warnings.warn(msg)
+    
+    def list_active_storms(self,basin='all'):
         
         r"""
         Produces a list of storms currently stored in Realtime.
         
+        Parameters
+        ----------
+        basin : str
+            Basin for which to return active storms for. Default is 'all'.
+        
         Returns
         -------
         list
-            List containing the storm IDs for currently active storms. Each ID has a Storm object stored as an attribute of Realtime.
+            List containing the storm IDs for currently active storms in the requested basin. Each ID has a Storm object stored as an attribute of Realtime.
         """
         
-        return self.storms
+        if basin == 'all':
+            return self.storms
+        
+        keys = []
+        for key in self.storms:
+            if self[key].basin == basin: keys.append(key)
+        
+        return keys
     
     def get_storm(self,storm):
         
@@ -571,3 +684,154 @@ class Realtime():
         #Return RealtimeStorm object
         return self[storm]
 
+    def plot_summary(self,domain='all',ax=None,cartopy_proj=None,save_path=None,**kwargs):
+        
+        r"""
+        Plot a summary map of ongoing tropical cyclone and potential development activity.
+        
+        Parameters
+        ----------
+        domain : str
+            Domain for the plot. Default is "all". Please refer to :ref:`options-domain` for available domain options.
+        ax : axes, optional
+            Instance of axes to plot on. If none, one will be generated. Default is none.
+        cartopy_proj : ccrs, optional
+            Instance of a cartopy projection to use. If none, one will be generated. Default is none.
+        save_path : str, optional
+            Relative or full path of directory to save the image in. If none, image will not be saved.
+        
+        Other Parameters
+        ----------------
+        two_prop : dict
+            Customization properties of NHC Tropical Weather Outlook (TWO). Please refer to :ref:`options-summary` for available options.
+        invest_prop : dict
+            Customization properties of active invests. Please refer to :ref:`options-summary` for available options.
+        storm_prop : dict
+            Customization properties of active storms. Please refer to :ref:`options-summary` for available options.
+        cone_prop : dict
+            Customization properties of cone of uncertainty. Please refer to :ref:`options-summary` for available options.
+        map_prop : dict
+            Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
+        
+        Notes
+        -----
+
+        The following properties are available for plotting NHC Tropical Weather Outlook (TWO), via ``two_prop``.
+
+        .. list-table:: 
+           :widths: 25 75
+           :header-rows: 1
+
+           * - Property
+             - Description
+           * - plot
+             - Boolean to determine whether to plot NHC TWO. Default is True.
+           * - days
+             - Number of days for TWO. Can be either 2 or 5. Default is 5.
+           * - fontsize
+             - Font size for text label. Default is 12.
+
+        The following properties are available for plotting invests, via ``invest_prop``.
+
+        .. list-table:: 
+           :widths: 25 75
+           :header-rows: 1
+
+           * - Property
+             - Description
+           * - plot
+             - Boolean to determine whether to plot active invests. Default is True.
+           * - linewidth
+             - Line width for past track. Default is 0.8. Set to zero to not plot line.
+           * - linecolor
+             - Line color for past track. Default is black.
+           * - linestyle
+             - Line style for past track. Default is dotted.
+           * - fontsize
+             - Font size for invest name label. Default is 12.
+           * - ms
+             - Marker size for invest location. Default is 14.
+
+        The following properties are available for plotting storms, via ``storm_prop``.
+
+        .. list-table:: 
+           :widths: 25 75
+           :header-rows: 1
+
+           * - Property
+             - Description
+           * - plot
+             - Boolean to determine whether to plot active storms. Default is True.
+           * - linewidth
+             - Line width for past track. Default is 0.8. Set to zero to not plot line.
+           * - linecolor
+             - Line color for past track. Default is black.
+           * - linestyle
+             - Line style for past track. Default is dotted.
+           * - fontsize
+             - Font size for storm name label. Default is 12.
+           * - fillcolor
+             - Fill color for storm location marker. Default is color by SSHWS category ("category").
+           * - label_category
+             - Boolean for whether to plot SSHWS category on top of storm location marker. Default is True.
+           * - ms
+             - Marker size for storm location. Default is 14.
+
+        The following properties are available for plotting realtime cone of uncertainty, via ``cone_prop``.
+
+        .. list-table:: 
+           :widths: 25 75
+           :header-rows: 1
+
+           * - Property
+             - Description
+           * - plot
+             - Boolean to determine whether to plot cone of uncertainty & forecast track for active storms. Default is True.
+           * - linewidth
+             - Line width for forecast track. Default is 1.5. Set to zero to not plot line.
+           * - alpha
+             - Opacity for cone of uncertainty. Default is 0.6.
+           * - days
+             - Number of days for cone of uncertainty, from 2 through 5. Default is 5.
+           * - fillcolor
+             - Fill color for forecast dots. Default is color by SSHWS category ("category").
+           * - label_category
+             - Boolean for whether to plot SSHWS category on top of forecast dots. Default is True.
+           * - ms
+             - Marker size for forecast dots. Default is 12.
+        """
+        
+        #Retrieve NHC shapefiles for development areas
+        shapefiles = get_two_current()
+        
+        #Retrieve kwargs
+        two_prop = kwargs.pop('two_prop',{})
+        invest_prop = kwargs.pop('invest_prop',{})
+        storm_prop = kwargs.pop('storm_prop',{})
+        cone_prop = kwargs.pop('cone_prop',{})
+        map_prop = kwargs.pop('map_prop',{})
+        
+        #Create instance of plot object
+        try:
+            self.plot_obj
+        except:
+            self.plot_obj = TrackPlot()
+        
+        #Create cartopy projection
+        if cartopy_proj is not None:
+            self.plot_obj.proj = cartopy_proj
+        else:
+            self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0) #0.0
+        
+        #Plot
+        ax = self.plot_obj.plot_summary([self.get_storm(key) for key in self.storms],
+                                        [self.get_storm(key).get_forecast_realtime() if self[key].invest == False else {} for key in self.storms],
+                                        shapefiles,dt.utcnow(),domain,ax,save_path,two_prop,invest_prop,storm_prop,cone_prop,map_prop)
+        
+        return ax
+        
\ No newline at end of file
